using System;
using System.Collections.Generic;
using System.Text;
using Modbus.Device;      
using System.IO.Ports;    
using System.Net.Sockets;
using System.Threading;
using System.IO;

namespace catchException
{
    class Program
    {
        static void Main(string[] args)
        {
            SerialPort serialPort = new SerialPort("COM1", 115200, Parity.None, 8, StopBits.One);
            serialPort.Open();
            ModbusSerialMaster master = ModbusSerialMaster.CreateRtu(serialPort);           
            master.Transport.ReadTimeout = 500;
            while (true)
            {
                try
                {
                    ushort[] register = master.ReadInputRegisters(1, 0, 10);
                    for (int index = 0; index < register.Length; index++)
                        Console.WriteLine(string.Format("register[{0}] = {1}", index, register[index]));
                }
                catch (Exception exception)
                {
                    //Connection exception
                    //No response from server.
                    //The server maybe close the com port, or response timeout.
                    if (exception.Source.Equals("System"))                        
                        Console.WriteLine(exception.Message);

                    //The server return error code.
                    //You can get the function code and exception code.
                    if (exception.Source.Equals("nModbusPC"))
                    {                
                        string str = exception.Message;
                        int FunctionCode;                       
                        string ExceptionCode;

                        str = str.Remove(0, str.IndexOf("\r\n") + 17);
                        FunctionCode = Convert.ToInt16(str.Remove(str.IndexOf("\r\n")));
                        Console.WriteLine("Function Code: " + FunctionCode.ToString("X"));

                        str = str.Remove(0, str.IndexOf("\r\n") + 17);
                        ExceptionCode = str.Remove(str.IndexOf("-"));
                        switch (ExceptionCode.Trim())
                        {
                            case "1":
                                Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> Illegal function!");
                                break;
                            case "2":
                                Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> Illegal data address!");
                                break;
                            case "3":
                                Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> Illegal data value!");
                                break;
                            case "4":
                                Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> Slave device failure!");
                                break;
                        }

                        /*
                           //Modbus exception codes definition
                            
                           * Code   * Name                                      * Meaning
                             01       ILLEGAL FUNCTION                            The function code received in the query is not an allowable action for the server.
                         
                             02       ILLEGAL DATA ADDRESS                        The data addrdss received in the query is not an allowable address for the server.
                         
                             03       ILLEGAL DATA VALUE                          A value contained in the query data field is not an allowable value for the server.
                           
                             04       SLAVE DEVICE FAILURE                        An unrecoverable error occurred while the server attempting to perform the requested action.
                             
                             05       ACKNOWLEDGE                                 This response is returned to prevent a timeout error from occurring in the client (or master)
                                                                                  when the server (or slave) needs a long duration of time to process accepted request.
                          
                             06       SLAVE DEVICE BUSY                           The server (or slave) is engaged in processing a longVduration program command , and the
                                                                                  client (or master) should retransmit the message later when the server (or slave) is free.
                             
                             08       MEMORY PARITY ERROR                         The server (or slave) attempted to read record file, but detected a parity error in the memory.
                             
                             0A       GATEWAY PATH UNAVAILABLE                    The gateway is misconfigured or overloaded.
                             
                             0B       GATEWAY TARGET DEVICE FAILED TO RESPOND     No response was obtained from the target device. Usually means that the device is not present on the network.

                         */
                    }

                }
                Thread.Sleep(1000);
            }
        }
    }
}
