﻿using System;
using System.Collections.Generic;
using System.Text;
//step1. reference nmodbuspc.dll, and using the namespaces.
using Modbus.Device;      //for modbus master
using System.Net;         //for tcp client
using System.Net.Sockets;

namespace holding_register
{
    class Program
    {
        static void Main(string[] args)
        {
            // connect to Modbus TCP Server
            string ipAddress = "127.0.0.1"; //Input WISE IP
            int tcpPort = 502;
            byte slaveID = 1;
            TcpClient tcpClient = new TcpClient(ipAddress, tcpPort);

            // create Modbus TCP Master by the tcp client
            //document->Modbus.Device.Namespace->ModbusIpMaster Class->Create Method
            ModbusIpMaster master = ModbusIpMaster.CreateIp(tcpClient);

            // rewrite the value of AO ch0 (40020~40021) by float
            //document->Modbus.Device.Namespace->ModbusMaster Class->WriteSingleRegister Method             
            ushort rewriteAddress = 20;
            ushort[] rewriteValue = new ushort[2];
            float[] floatData= new float[1] { 123.456f };   //Value of AO ch0
            Buffer.BlockCopy(floatData, 0, rewriteValue, 0, 4);
            for (ushort index = 0; index < 2; index++)
                master.WriteSingleRegister(slaveID, (ushort)(rewriteAddress+index), rewriteValue[index]);
          
            // read the holding register 20~29 (40020~40029, AO ch0 ~ ch4, float type) of the WISE module.
            //document->Modbus.Device.Namespace->ModbusMaster Class->ReadHoldingRegisters Method          
            ushort startAddress = 20;
            ushort numOfPoints = 10;
            master.Transport.ReadTimeout = 1000;

            try
            {
                ushort[] register = master.ReadHoldingRegisters(slaveID, startAddress, numOfPoints);
                float[] floatData2 = new float[register.Length / 2];
                Buffer.BlockCopy(register, 0, floatData2, 0, register.Length * 2);
                for (int index = 0; index < floatData2.Length; index++)
                    Console.WriteLine(string.Format("AO[{0}] = {1}", index, floatData2[index]));
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
            }

            Console.WriteLine("Press enter to exit....");
            Console.ReadLine();
        }
    }
}

