using System;
using System.Collections.Generic;
using System.Text;
//step1. reference nmodbuspc.dll, and using the namespaces.
using Modbus.Device;      //for modbus master
using System.Net;         //for tcp client
using System.Net.Sockets;
using System.Threading;

namespace input_register
{
    class Program
    {
        static void Main(string[] args)
        {
            // connect to Modbus TCP Server
            string ipAddress = "127.0.0.1"; //Input WISE IP
            int tcpPort = 502;
            byte slaveID = 1;
            TcpClient tcpClient = new TcpClient(ipAddress, tcpPort);

            // create Modbus TCP Master by the tcp client
            //document->Modbus.Device.Namespace->ModbusIpMaster Class->Create Method
            ModbusIpMaster master = ModbusIpMaster.CreateIp(tcpClient);

            // read the input register 20~29 (30020~30029, AI ch0 ~ ch4, float type) of the WISE module.
            //document->Modbus.Device.Namespace->ModbusMaster Class->ReadInputRegisters Method            
            ushort startAddress = 20;
            ushort numOfPoints = 10;
            master.Transport.ReadTimeout = 1000;
            try
            {
                ushort[] register = master.ReadInputRegisters(slaveID, startAddress, numOfPoints);
                float[] floatData = new float[register.Length / 2];
                Buffer.BlockCopy(register, 0, floatData, 0, register.Length * 2);
                for (int index = 0; index < floatData.Length; index++)
                    Console.WriteLine(string.Format("AI[{0}] = {1}", index, floatData[index]));
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
            }
            Console.WriteLine("Press enter to exit....");
            Console.ReadLine();
        }
    }
}
